function [LB, UB, D, opt_f, err,quantis] = get_fun_info(fun,D0)
% Get the lower bound, upper bound, dimension, and optimal value for function # 'fun'
opt_f = -inf;
quantis=[];
  	err=0;
   	opt_f=0;
 
switch fun

	case {-2, -3} % Line, Piece-wise linear
	D=1;
	LB=0;
	UB=1;
	
	case -1 % Plateau
		if exist('D0','var') && ~isempty(D0) 
		 D=D0; 
		else D=2; % Default value
		end

		LB=zeros(1,D);
		UB=ones(1,D);
		  
		case 0 % Parabol, Paraboloid
			if exist('D0','var') && ~isempty(D0) 
		 D=D0; 
		else D=2; % Default value
		end
		LB= zeros(1,D);
    UB= 3*D*ones(1,D);
      
    case 1 % Alpine. Minimum (1,2,...,D)
    	if exist('D0','var') && ~isempty(D0) 
		 D=D0; 
		else D=2; % Default value
		end
		
    	LB= zeros(1,D);
    	UB= 4*D*ones(1,D);
      err =0; % 1e-4;
      opt_f=0;
      
    case 2 % Tripod. Minimum (0, -50)	
       D=2;
    	UB= 100*ones(1,D);
    	LB= -UB;
      
   case 3 % Gear train
   % Min 2.70086e-12
   % (16 19 43 49)
   	D=4;
   	ones4=ones(1,4);
   LB=12*ones4;
   	UB=60*ones4;
   	quantis=ones4;, 
   	  % Actually, when looking at f (see func.m), it is easy to see that 
   %	  for the minimum we must have 
   %		x1, x2 in {13,14, ...,39}, and
   %		x3, x4 in {17, 18, ..., 60}
 	
   	case 4 % Pressure vessel
   	% (constraints as penalties, see get_fun_info.m)
   	% 100 runs, FEmax=4500
   	%  => f( 0.8125    0.3750   40.5252  197.1576)=5942.739
   	D=4;
   	q=0.0625;
   	LB=[q q 10 10];
   	UB=[99 99 200 200];
   	err=0;
   	quantis=[q q 0 0];
   	opt_f=0;
   	
   	case 6 % Flash
   	 D=1;
   LB=0;
   UB=1;
   	
   	case 7 % Comb
   	 D=1;
   LB=0;
   UB=10;
   	err=0;
   	opt_f=0;
 
   	case 8 % Brush
   	D=2;
   LB=[0 0];
   UB=[10 10];
   	err=0;
   	opt_f=0;
   	
   	case 9 % TSP 6 nodes
   	D=1;
   	LB=123456; % code for (1 2 3 4 5 6)
   	UB=654321;
   	quantis=1;
   	
   	case 10 % Canyon
   	D=1;
   LB=0;
   UB=10;
   	err=0;
   	opt_f=0;
   	
   	case 11 % Six-hump Camel Back
   	D=2;
   	LB=[-3,-2];
   	UB=-LB;
   	 	err=0;
   	opt_f=0;
   	
    otherwise
    if fun >1000 && fun <2151 % Lennard-Jones
    nbAtoms=fun-1000;
    	% 	Best results in the literature (2015)
   	% http://doye.chem.ox.ac.uk/jon/structures/LJ/tables.150.html
   	% * means: found by APS with automatic FEmax, and at most 100 runs
   	%		nbAtoms  Min
   	%		3*				-3 (proved) 
   	%		4*				-6 (proved) 
   	%		5*		 		-9.103852415707552 (proved) 
   	%		6*		 		-12.712062 
   	% ...
   	%		10*				-28.422532
  	% ...
   	% ...
   	%		20*				-77.177043
   	%		30*			-128.286571
   	%		31			-133.586422 	

   	D=3*nbAtoms;
   	LB=-2*ones(1,D); UB=-LB;
   	err=0;
   	op_f=0;
    
    else
        error('function %i not implemented', fun)
		end
end

if length(quantis)==0
	quantis=zeros(1,D);
end
end
