function  [xBest, fBest,FEs, intermed,pos,cost,tri]= ...
  TMO(funSet, fun, FEmax, opt_f,FuncAddr,LB,UB,init, ...
  around, distrib, morph, rnk,reTriang, estime,opp, ...
  FEintermed, quantis,verbose,plotFacet,plotStep,plotAnim, reuseInit,allPos)
%{
	Total Memory Optimiser
 Proposed by: Maurice Clerc
 Version: 2016-04-26
 
 Warning: must be launched by TMO_Monte
  %}
  
  %{
	Inputs
	------
		See TMO_Monte
	
	Outputs
	-------
	xBest			Best position found
	fBest			Value of this position
	FEs				Number of function evaluations
	intermed	Intermediate results (option)
	pos				All current positions (for tests)
	cost			Values of these positions (for test)
	tri				Last triangulation, on these positions (for tests)
		
   Principles of the algorithm:
   1) All evaluated positions are memorised
   2) After each evaluation, a surrogate function is defined by triangulation
   3) Next new position is drawn according to the custom-made RPG method
  %}
  global verbose
  global tetra tetraEstim 
  
  toQuantis=sum(quantis)>0; % If true, it means some coordinates are discrete
  
  intermed=[]; % To save intermediate results, if asked
  
  % Initial positions
  
  posInit=initPos(init,quantis);
  [S0,D]=size(posInit);
  
  if S0>FEmax
  	fprintf('\n FEmax (%i) is too small to initialise %i positions',...
  	FEmax, S0);
  	error(' ')
  end
  
    fprintf( '\nInitial population size %i, dimension %i ',S0,D);
  
  % First evaluations
  best=1;
  FEs=0;
  
  for s=1:S0
    [posReal(s,1:D),costInit(s),FEs]=...
      fEval(posInit(s,:), fun, opt_f,FuncAddr,FEs, LB, UB,quantis);
  end % for s=1:S0
  
  %  'best ' = rank of the best particle
 [costS Ind]=sort(costInit);
 if opp==1 best=Ind(1); else best=Ind(S0); end
  
  fprintf( '\nBest after init: %f ',costInit(best));
  
  fBest=costInit(best);
  
  S=S0;
  pos=posInit;
  cost=costInit;
  [m,best]=min(cost);
  posBest=pos(best,:);
  
  if D==1
    [pos,rPos]=sort(pos);
    for n=1:S
      cost(n)=costInit(rPos(n));
    end
  end

  % Triangulation
  if reuseInit % Re-use saved triangulation
    %															Of course, more may have been saved, for
    %															higher dimensions (see initSave.m)
    %															In that case, modify Dmax accordingly
    % This should be used with reInit>1, in particular reInit=Inf,
    % to reduce the computing time

    tri=initLoad(D); % TO CHECK: sorted positions?
   
  else
    tri=triang(pos);
  end
 
  % Estimation of the values of the representatives of the  'triangles '
  [posEstim,costEstim]=facetEstim(pos,cost,tri,estime,opp);
  
  % Saves for possibly re-use if no retriangulation
 % tetra=tri;
 % tetraEstim=[posEstim costEstim'];

  %--------------------------------- Evaluations loop
  while FEs<FEmax
    
    % Optional plot
    if plotFacet>0  && mod(FEs,plotStep)==0
      x=pos(:,1)';
      if D==1 y=[]; else y=pos(:,2)'; end
      plotDelaunay(x,y,cost',LB,UB,plotFacet,tri,plotAnim,init,reTriang) ;
     end
      
      % Creates a new position, according to the probability distribution,
      % which is itself "similar" to the surrogate function,
      % defined by triangulation
      
      [posNew,triRank]=customRPG(tri,opp,pos,posEstim,...
        cost,costEstim,morph,around,rnk,reTriang,allPos,...
        toQuantis,quantis);
         
      % Evaluates the new position, and adds it to the population
      S=S+1;
      pos(S,1:D)=posNew;
      [posReal(S,1:D),cost(S),FEs]=fEval(posNew, fun, opt_f,FuncAddr,FEs, LB, UB,quantis); 
   
      %  ... if best than the best, replaces it
      if (opp==1 & cost(S)<cost(best)) | (opp==0 & cost(S)>cost(best))
        best=S;
        posBest=pos(best,:);
          if verbose>0 
  					fprintf('\n cost(%i) = %f',S,cost(S))
  				end 
      end
      
      if D==1  
        [pos,rPos]=sort(pos);  % Sorts the positions and the values
  			cost=cost(rPos);
 
  			 	% Complete re-triangulation
        	tri=triang(pos); % Redefines the intervals     
       		[posEstim,costEstim]=facetEstim(pos,cost,tri,estime,opp);
        	if opp==1   [m,best]=min(cost);
        		else [m,best]=max(cost); 
        	end
        
      else	 % D>1
      
      if triRank==0
        % Finds the "triangle" that contains posNew
        % so that complete re-triangulation is not needed 
        [nbTri,~]=size(tri);
 
    		for v=1:nbTri
    			 vert=pos(tri(v,:),:);
      		[isInside,~]=inside(posNew,vert);
      		if isInside
      			triRank=v;
   				   break;
      		end
      	end
      end    
        
        if  mod(FEs,reTriang)==0 || triRank==0 % Rebuilds the triangulation
          % Complete re-triangulation
          tri=triang(pos);
          [posEstim,costEstim]=facetEstim(pos,cost,tri,estime,opp);
        else
          % Updates the triangulation. pos(S) is the new point
          % pos(S) is inside a known facet
          % Removes this facet
          facet=tri(triRank,:);
          tri(triRank,:)=[];
          posEstim(triRank,:)=[];
          costEstim(triRank)=[];
          
          % Builds the sub-facets
          [nTri,D1]=size(tri);
          
          for n=1:D1  % There are D+1 subfacet
            nTriN=nTri+n;
            for k=1:D
              tri(nTriN,k)=facet(1+mod(k+n-2,D1));
            end
            tri(nTriN,D1)=S;
          end   
          
          [nTri,D1]=size(tri);
          [posEstimPlus,costEstimPlus]=...
            facetEstim(pos,cost,tri(nTri-D:nTri,:),estime,opp); % D+1 points
          posEstim=[posEstim;posEstimPlus];
          costEstim=[costEstim costEstimPlus];
          
          
        end % if  reTriang==1 || triRank==0
      end % if D==1
      
      % Saves intermediate results
      Ind=find(FEintermed<=FEs);
      indL=length(Ind);
      interm=max(1,indL);
      intermed(interm)=min(cost);
      
    end % while FEs<=FEmax  --------------------------------------------------
  
    xBest=posReal(best,:);
    
    if (opp==1 & cost(best)<fBest) | (opp==0 & cost(best)>fBest)
      fBest=cost(best);
    end
    
    % Displays results
    fprintf( '\n Best => %e  ', cost(best));
    if verbose>0
      fprintf( '( ');
      for d=1:D
        fprintf( ' %f ',posReal(best,d));
      end
      fprintf( ') ');
    end
    
    fprintf( '\nTotalMem ');
    fprintf( '\nProblem %i, dimension %i ',fun,D);
    fprintf( '\n  %i evaluations ',FEmax);
    fprintf( '\n Population  %i => %i ',S0,S);
    
    fprintf( '\n Best position:\n ');
    for d=1:D fprintf( '%e,  ',xBest(d)); end
    
    if opp==1   fprintf( '\n Minimum  '); end
    if opp==0   fprintf( '\n Maximum  '); end
    fprintf('%e',fBest);
    
    if distrib>1   % Computes more than one point, for further tests
      for n=1:distrib
        [posNew,triRank]=customRPG(tri,opp,pos,posEstim,...
          cost,costEstim,morph,around,rnk);
        posDistrib(n,1:D)=posNew; % Just for further analysis if n>1
      end
    else
      posDistrib=[];
    end % if distrib>1 

end
%================================================================= facetEstim
function [posEstim,costEstim]=facetEstim(pos,cost,tri,estim,opp)
% For each facet of tri, defines and evaluates a position that is supposed 
% to represent it

[nbPos,D]=size(pos);
[nbTri,D1]=size(tri);

switch estim
  case 0 % center
    for t=1:nbTri
      vert=pos(tri(t,:),:);% D+1 nodes of the current facet
      posEstim(t,1:D)=mean(vert); % Middle
      costEstim(t)=mean(cost(tri(t,:)));
      
    end % for t=1:nbTri
    
  case 1 % weights according to the ranks
    for t=1:nbTri
      vert=pos(tri(t,:),:);% D+1 nodes of the current facet
      val=cost(tri(t,:));
      if opp==1   % decreasing order if minimisation
        [valSorted,valRank]=sort(val,  'descend');
      else % increasing order if maximisation
        [valSorted,valRank]=sort(val, 'ascend')
      end
      s=sum(valRank);
      weight=valRank/s;
      	for d=1:D			
      		posEstim(t,d)=weight*vert(:,d);   		
      	end
      costEstim(t)=weight*valRank';
    end % for t=1:nbTri
    
  case 2 % weights according to the values
    for t=1:nbTri
			 vert=pos(tri(t,:),:);
      val=cost(tri(t,:));
      if opp==1   % decreasing order is minimisation
        val=max(val)-min(val)+val;
      end
      s=sum(val);
      weight=val/s;
			
				for d=1:D			
      		posEstim(t,d)=weight*vert(:,d);   		
      	end
    
      costEstim(t)=weight*val';
    end % for t=1:nbTri
    
end %switch estim
end


