function [isInside,C]=inside(x,vert)
% Checks whether x is inside the polyhedron or not
% defined by its D+1 vertices
% yes => isInside=true
% no => isInside=false
% method: computes the normalised barycentric weights (C)
%     they must be all >=0

[~,D]=size(x);
[nVert,Dv]=size(vert);
if Dv~=D
  fprintf('\ninside. Error: dimension of x is %i',D);
  fprintf('\n   as the dimension of the vertices is %i',Dv);
  
end

if nVert~=D+1
  fprintf('\ninside. The number of vertices is %i',nVert);
  fprintf('\n  It must be %i',D+1);
  error(' ')
end

for v=1:D+1
  A(1:D,v)=vert(v,:)';
end
A(D+1,1:D+1)=ones(1,D+1);

B=x';
B(D+1)=1;

C=linsolve(A,B); % AC-B=0

isInside=true;
for c=1:D+1
  if C(c)<0
    isInside=false;
    break;
  end
end

end

