function TMO_Monte (funSet, funArray, budgetArray, Darray,paramSet)

%{
 Used to launch TMO:
 Total Memory Optimiser
 Proposed by: Maurice Clerc
 Version: 2016-04-26
 
 Reference:
 Clerc, Maurice. “Total Memory Optimiser: Proof of Concept and Compromises,”
 							 March 2016. https://hal.archives-ouvertes.fr/hal-01294101.

 
 WARNING: this is a "proof of concept" code.
 					It is extremely slow as soon as the dimension of the problem
 					is greater than 4
 					In that case, you should use compromises (see init and reTriang)

	Inputs
	------
 funSet = 	1 real-world CEC 2011 benchmark
						2	CEC 2005 benchmark (artificial problems)
						3 MyFunctions 
										
 For each set, see get_fun_infos.m and func.m in the corresponding directory
 for details.
	----									
 funArray = array of function codes
         
 Note: for Lennard-Jones (in MyFunctions), the function code is 1000+nb_of_atoms
 For example, to run it with 5 atoms, the command is
 TMO_Monte(3,1005) with automatic limited budget (13400 in this case)
 or
 TMO_Monte(3,1005,20000) with at most 20000 fitness evaluations.
 
 It means you can test for different numbers of atoms with a unique command like
 TMO_Monte(3,[1010 1020 1030])
 and the results are saved in different files
 WARNING: As the dimension of the problem is 3*nb_of_atoms, 
 					TMO is too slow on this problem if you do not use
 					a very powerful computer
	----
 	budgetArray = budget in terms of number of function evaluations (FEs)
 	
 	----
 	Darray = dimension of each problem. ONLY for CEC 2005

 Example for all the CEC 2011 functions:
 funArray=[1:10 111:119,1110,12,13];
 Warning: apparently, there is a bug in the evaluation for 1110
					(constraint weigth set to zero). Therefore, it is identical to 118
	budgetArray=100*[60	134	24 24	134	134	109	64 275 84	268	360	60	88 94 154	289	240	240	240	124	114];
 TMO_Monte(1,funArray, budgetArray)

 If you want to use the default "limited budget" array, just
 TMO_Monte(1,funArray)

 For 150,000 FE on n functions of CEC2011
 TMO_Monte(1, [list of n functions], 150000*ones(1,n));
 
 			example = integer.  See TMO_example.m for details.
 								Used to run predefined examples
 								WARNING. It overloads the parameter values 
 													that are defined below
 	----
 	paramSet = n, in {1, 2, ..., 12}. Just for predefined examples.
 							See TMO_example.m and TMO_param.m
 													 
%}

global  initial_flag	% Needed for CEC 2005 benchmark
initial_flag = 0;

global verbose
global runPlot nExper
global F videoFile frame  % For movie (plotAnim option)
global tetra tetraEstim

warning 'off'

fprintf('\n=========== TMO_Monte\n')


%----------------------------------------------------------------------- Options
nbExperiment=100; %Number of experiments.
%{
			An "experiment" is the complete use of the "budget"
 			For fair comparisons, for a given problem, the best result
				of each experiment is recorded, and the _distribution_ of these results
				gives us a good idea of the performance of the method on this problem
				This distribution itself can be plotted, or at least summarised
				by (mean, std. dev, median).
				Note that for a correct estimation, nbExperiment should be
				at least 50. See the results in the files
			 Recap.txt, Table.txt, Exper.txt, Intermed.txt
				You may also apply later compareResults() for a comparison based on
				a user-defined quality function.
%}

init=2; % How to initialise
%{
   0 => at random
   1 => corners. Warning: just for test. Only corners are not enough
   												to start a correct triangulation
   2 => corners+center (pure TMO)
   3 => corners + random
   4 => random + center + one corner at random. APS formula to define the
   			population size
   5 =>  the same, but with CMA-ES formula  
%}

reTriang=1;
%{
	1 => the triangulation is entirely rebuilt each time a new position
				is added (pure TMO)
	n => the triangulation is rebuilt every n new positions.
				Else, it is just locally modified.
				Warning: the result is usually not a correct Delaunay's triangulation
	Inf =>* no retriangulation at all, just local modifications

%}

reuseInit=false; 
%{
		true => reuse the saved triangulation (in ./initSaved/. See initSave.m)
		Warning: be sure it has been saved for the dimension of the problem.
		Valid ONLY for init=2 and D>1
%}
Dmax=6; % Maximum dimension for which the initial triangulation has been saved

%--------------------------------------- Usually not to modify, except for tests
allPos=true; % 	true =>* Takes all positions into account (known and estimated)
%  							false => only estimated positions
  
around=1; % Useful only if allPos=true
%{
   around: how to look "around" a known position
   0 Rudimentary method, dimension wise  (not bad at all)
 * 1  Along each dimension, finds the nearest estimated point
       and "exploits" at random the interval
 	11  Along each dimension, finds the nearest point (estimated or known)
       and "exploits" at random the interval
 	12 As 11, but with non-uniform randomness
 	*13 Variant of 12
   131
 	*	14 Variant of 13
 
?  2  Finds the nearest point, and "exploits"
       inside a D-rectangle defined by it and the position n
? 	3  Invariant by rotation. Defines a polyhedron
 				and draws at random inside it
	*4  Finds a facet (at random) it belongs to
 		Chooses one at random
 		Selects a point at random "between" the position
       and the representative of the facet
  41 Finds the first facet it belongs to.
 			Selects a point at random "between" the position
       and the representative of the facet
%}

distrib=1;
%{
  n => compute n new points according to the estimated distribution.
  			Usually n=1 (default value)
        More points ONLY for test purpose (drawing the estimated distribution)
%}

morph=1; %* 1.   In ]0 2[

%{
   <1 to speed up the convergence (but risky)
   >1 to slow down the convergence
%}

rnk=1;
%{
   0 => uses f values
  * 1 => uses ranks (same behaviour for equivalent functions)
%}

estime=0; % How to estimate a facet
%{
   0 =>* center of the facet
          1 => weights of the vertices of the facet= ranks of the values
              (decreasing order if minimisation)
          2 => weights = values
%}
opp=1; % 0 => looking for a maximum
% 			 1 => looking for a minimum

%-------------------------------------------------- Plot options
runPlot=0; % Which experiment (run) to plot
plotLand=false; % true => plots the landscape (at most two dimensions)
plotFacet=0;  %  Plots the facets from time to time (see plotStep)

%									1 => plots just the triangulation on the (x,y) plane
%									2 => plots the facets of the triangulation (max. 2D)
%									3 => both
plotStep=0; % n => plots intermediate triangulation, every n evaluations
%								  	active only if plotFacet>0
%							0 => no intermediate plot (you may set plotLast=1)
plotLast=0; % 1 => plots the last triangulation of each experiment (run)
plotDistrib=false; % true => plots the distribution of the best results
%											Useful only if nbExperiment is not too small
%											(typically >=50)
nbBins=0; %10; % Number of bins for the CDF plot

% --------------------Options for animation
plotAnim=false; % true => instead of different plots, keeps just one figure
%							Should be used just with plotFacet=1, and plotStep>0
%							The video is also saved on a .avi file

%------------------------------------------------- Other options
savePos=false; % To save all positions on a file
signature=false; % To plot all  positions (2D). 
%										Mainly used with the function Plateau
%										(typically 10 runs of 1000 evaluations)
verbose=1; % 0 => almost no display
% 					 1 => more information


%----------------------------------------------------------- Predefined examples
%  WARNING. It overloads most of above parameter values

if exist('paramSet','var')

[nbExperiment,init,reTriang,reuseInit,allPos,around,distrib,morph,...
					rnk,estime,opp,runPlot,plotLand, plotFacet,plotStep,plotLast,...
					plotAnim,plotDistrib,nbBins,verbose]=TMO_param(paramSet);
end

%-------------------------------------------------------------------------------

%{
 Problems
 (see get_fun_info.m and func.m in the directory of each set of problems)
 --------
 
 funSet 3
 --------
 -1 Plateau (2D)
 1 Alpine (see get_fun_info for dimension)
 2 Tripod (see get_fun_info for dimension)
 3 Gear Train (4D)
 4 Pressure Vessel (4D)
 6 Flash (1D)
 7 Comb (1D)
 8 Brush (2D)
 1005 Lennard-Jones 5 atoms (15D)
 
%}

%------------------------------------------------- Checks parameters and options
% -------- First set of tests
  OK=1;
  fA=length(funArray);
if exist('budgetArray','var')
  bA=length(budgetArray);
  
  if bA>0 && bA ~= fA
    fprintf('\nERROR: Number of budgets (%i) and functions (%i) are different\n',...
      bA,fA);
      OK=0;
  end
 end 
 
  if exist('Darray','var')
  DA=length(Darray);
  	if DA ~=fA
  	fprintf(...
  	'\nERROR: number of dimensions (%i) and function (%i) are different\n',...
  	DA,fA);
  	OK=0;
  	end
  end

if funSet==2  % For CEC 2005, you must give the dimension
  if ~exist('Darray','var') 
    fprintf(...
    '\nERROR: For CEC 2005, you must give the dimension (Darray)\n')
  end
  OK=0;
end


if OK==0 return; end

% -------- Second set of test
OK=1;

if ~ismember(around, [0:4 11:14  131 41])
  fprintf('\n Error. Please check the value of the around parameter');
  OK=0;
end

if morph<=0 || morph>=2
  fprintf('\n Error.  morph must be in ]0, 2[');
  OK=0;
end

if ~ismember(rnk, [0 1])
  fprintf('\n Error.  rnk must be 0 or 1');
  OK=0;
end

if ~ismember(estime, [0 1 2])
  fprintf('\n Error.  estime must be 0 or 1 or 2');
  OK=0;
end

if ~ismember(opp, [0 1])
  fprintf('\n Error.  opp must be 0 or 1');
  OK=0;
end

if ~ismember(init, [0:5])
  fprintf('\n Error.  Acceptable values for init are {0 1 2 3 4 5} ');
  OK=0;
end

if nbBins>=nbExperiment
  fprintf('Warning. nbExperiment is too small compared to nBins');
  OK=0;
end

if runPlot>nbExperiment
  fprintf('\n Error. runPlot must be smaller than nbExperiment');
  OK=0;
end

if plotAnim && plotStep==0
  fprintf('\n Error. For animation, you must have plotStep>0');
  OK=0;
end

%{
if reuseInit==1 && init~=2
	fprintf('\n Error. You can reuse the initial triangulation only with init=2');
	OK=0;
end
%}

fprintf('\n');
if OK==0 return; end


%---------------------------------------------Prepares files to save the results
for funct = 1:length(funArray)
  fun=funArray(funct);
  FileName{funct} = ['./RESULTS/TMO ', num2str(funSet), '[',num2str(fun),']'];
  
  fid(funct) = fopen(FileName{funct}, 'w');
  fprintf(fid(funct),' around  distrib morph rnk estime reTriang runPlot');
  fprintf(fid(funct),'\n  %i  %i %f %i %i %i %i \n', ...
    around, distrib, morph, rnk, estime, reTriang, runPlot);
  
  FileNameT{funct}=[FileName{funct} 'Table.txt'];
  fidT(funct) = fopen(FileNameT{funct}, 'w');
  fprintf(fid(funct),' around distrib morph rnk estime reTriang runPlot');
  fprintf(fid(funct),'\n  %i %i %f %i %i %i %i  \n', ...
    around, distrib, morph, rnk, estime, reTriang, runPlot);
  
  fprintf(fidT(funct),'\nProblem Best Median Mean Std Worst time/run \n');
  
  FileNameE{funct}=[FileName{funct} 'Exper.txt'];
  fidE(funct) = fopen(FileNameE{funct}, 'w');
  fprintf(fid(funct),' around distrib morph rnk estime reTriang runPlot');
  fprintf(fid(funct),'\n  %i  %i %f %i %i %i %i \n', ...
    around, distrib, morph, rnk, estime, reTriang, runPlot);
  fprintf(fidE(funct),'\nDistribution of the best results');
  
  FileNameI{funct}=[FileName{funct} 'Intermed.txt'];
  fidI(funct) = fopen(FileNameI{funct}, 'w');
  fprintf(fid(funct),' around distrib morph rnk estime reTriang runPlot');
  fprintf(fid(funct),'\n  %i %i %f %i %i  %i %i \n', ...
    around, distrib, morph, rnk, estime, reTriang, runPlot);
  
end

if ~exist('budgetArray','var') bdA=[]; else bdA=budgetArray;end
if ~exist('Darray','var') Da=[]; else Da=Darray; end
fName=['./RESULTS/TMO ', num2str(funSet), '[', num2str(funArray),'][', ...
 num2str(bdA),'][',num2str(Da),'] ', 'Recap.txt'];
 
fidRecap=fopen(fName,'w');
fprintf(fidRecap,' around distrib morph rnk estime reTriang runPlot');
fprintf(fidRecap,'\n  %i  %i %f %i %i  %i %i \n', ...
  around, distrib, morph, rnk, estime, reTriang, runPlot);
fprintf(fidRecap,'Function Best Median Mean Std_dev Worst time/run \n');

%-------------------------------------------------Defines the repertories to use
[GetFunInfoAddr, FuncAddr]=Addr(funSet);

%-------------------------------------------------------------------------------
nbFunc=length(funArray);
coeff=2450;	%  For "Limited budget" approach
%--------------------------------------------------------- Loop on functions-- 1
  
for funct = 1:nbFunc

		xTot=[];% For later tests (signature, for example)
		fTot=[]; % For later test (entropy, for example)

  frame=0; % Just in case animation is used
  if plotAnim % File where to save the animation
    videoFile=VideoWriter([FileName{funct} '.avi']);
    videoFile.FrameRate=3; % Can be modified
    open(videoFile)
  end
  
  fun=funArray(funct);
  
  switch funSet
    case 2 % CEC2005
      D=Darray(funct);
      [LB, UB, opt_f, ~] = GetFunInfoAddr(fun,D);
      quantis=zeros(1,D);
      
    case 3
    	if exist('Darray','var') 
    		D0=Darray(funct);
    	else D0=[];
    	 end

      [LB, UB, D, opt_f, ~,quantis] = GetFunInfoAddr(fun,D0);
      
    otherwise
      [LB, UB, D, opt_f, ~] = GetFunInfoAddr(fun);
      quantis=zeros(1,D);
  end
  
  if reuseInit==1 && D>Dmax
		fprintf('\n Error. No saved triangulation for dimension %i \n', D );
	return
	end
  
  if D>Dmax
    fprintf('\nWARNING. With dimension %i, the process may be very long',...
      D);
    fprintf('\n with some functions. You can abort it by Ctrl C');
  end
  
  
  % Normalises quantis
  if ~isempty(quantis)
    quantis=quantis./(UB-LB);
  end 
  
		%---------------------Plots the landscape (only for D=1 or 2)
	if plotLand
  if D<3
    figure('OuterPosition', [0 500 600 500], 'Name','Landscape','NumberTitle','off')
    switch D
      case 1
      	step=200;
        xNorm=0:1/step:1;
             
        for n=1:length(xNorm)
          [x(n),f(n)]=fEval(xNorm(n), fun, opt_f,FuncAddr,0, LB, UB,quantis);
        end
             
        plot(x,f,'red');
        
      case 2
        step=50;
        xNorm=0:1/step:1;
        yNorm=xNorm;
        
        for m=1:length(xNorm)
          for n=1:length(yNorm)
            [x(m,n,:),f(m,n)]=fEval([xNorm(m) yNorm(n)], fun, opt_f,FuncAddr,0,... 
            LB, UB,quantis);
          end
        end
        %fprintf('\n%f',std(reshape(f,[numel(f),1])))

        surf(x(:,:,1),x(:,:,2),f);
    end % switch D
   else
   fprintf('\n Sorry, I can not plot the landscape for dimension > 2\n')
   end % if D<3
  end % ---------------------------------------------if plotLand
  
  if plotFacet>0
  	if D<3
				titleFig=['init=',num2str(init), '   reTriang=', num2str(reTriang)];
  	 	figure('OuterPosition', [700 500 600 500], 'Name',titleFig,...
  	 	'NumberTitle','off')
  	else
  	   fprintf('\n Sorry, I can not plot the facets for dimension > 2\n')
  	end
  end
 
  fprintf('\nProblem %i, dimension %i',fun,D);
  
  if exist('budgetArray', 'var')
    FEmax=budgetArray(funct);
  else
    FEmax=100*floor(coeff*sqrt(D)/100);
  end
  
  FEintermed=floor(FEmax*(1:30)/30); % Intermediate FE values for which
  %																	results will be saved on a file
  nbIntermed=length(FEintermed);
  for interm=1:nbIntermed
    fprintf(fidI(funct),'%i ',FEintermed(interm));
  end
  
  rng(123456789); % Initialise the RNG for reproducible results
  %  RandSeed = round(sum(clock)); % For unreproducible results
  %             (except just statistically)
  
  % Warm up the RNG
  for t=1:10000	rand;	end
  
  % Saves search space
  fprintf(fid(funct),'\n');
  for d=1:D
    fprintf(fid(funct),'%1.4e ',LB(d));
  end
  fprintf(fid(funct),'\n');
  for d=1:D
    fprintf(fid(funct),'%1.4e ',UB(d));
  end
  
  %------------------------------------------------------------------- Warnings

if plotFacet>0  && ~plotAnim
	nbFig=FEmax/plotStep;
	fprintf('\n WARNING. You will plot at least %i figures', nbFig);
	answer=input('\n Is is really what you want?  y/n  ','s');
	if answer~='y' 
		return; 
	end  
 
 end 
  %---------------------------------------------------------- Experiments -- 1.1
  xBest=zeros(nbExperiment,D); % Reserves memory, just to speed up a bit
  fExper=Inf*ones(1,nbExperiment);
  t=zeros(1,nbExperiment);
  tic; % Starts the watch
  
  for nExper=1:nbExperiment
    fprintf('\n--------------------- Experiment %i/%i',nExper,nbExperiment);
    
    fprintf('\nSet %i, function %i, dimension %i, FEmax %i',funSet, fun, D, FEmax);
    
    [xBest(nExper,1:D), fExper(nExper),FEs, intermed,pos,cost,tri]= ...
      TMO(funSet, fun, FEmax, opt_f,FuncAddr,LB,UB,init, ...
      around, distrib, morph, rnk,reTriang, estime,opp, ...
      FEintermed, quantis,verbose,plotFacet,plotStep,plotAnim,reuseInit,allPos);
    
    intermedNb=length(intermed);
    % Complete the intermediate values
    for i=2:intermedNb
      if intermed(i)==Inf intermed(i)=intermed(i-1); end
    end
    intermedMin(1:intermedNb,nExper,funct)=intermed;
    
    % Displays current best result
    fprintf('\n								Min min %1.6e',min(fExper));
    
    % Saves detailed results (best value, best position)
    fprintf(fid(funct),'\n%i %i %1.6e ',nExper, FEs, fExper(nExper));
    fprintf(fid(funct),'\n');
    for d=1:D
      fprintf(fid(funct),'%1.6e ',xBest(nExper,d));
    end
    
    if plotLast>0  %  Plots the last triangulation (1D or 2D) 
      
      switch D
        case 1    	
          [pos Ind]=sort(pos);
          cost=cost(Ind);
          plot(pos,cost);
          
        case 2
          x=pos(:,1)'; y=pos(:,2)';
          plotDelaunay(x,y,cost,LB,UB,plotFacet,tri,false,init,reTriang)
      end
    end % if plotLast>0
    
    if signature || savePos 
    	xTot=[xTot; pos];
    end
    
    if savePos
    	fTot=[fTot cost];
    end
    
  end % for nExper=1:nbExperiment --------------------- End of experiments (1.1)
  
	 if signature  
  % Plots signature (normally used only for function Plateau (signature) 
  figure
plot(xTot(:,1),xTot(:,2),'LineStyle','none','Marker','o','MarkerFaceColor', 'blue')
[N,~]=size(xTot);
fprintf('\n nb of points %i', N); 
end
 
  timeRun=toc/nbExperiment; % Stops the clock, and computes the mean time
  fprintf('\n time/run  %1.6e',timeRun);
  % -------------------------------------------------------------- Saves results
  
  % Saves a table of best results, for later use (for example in a spreadsheet)
  fprintf(fidE(funct),'\nFunction_%i',fun);
  for nExper=1:nbExperiment
    fprintf(fidE(funct),'\n%1.6e',fExper(nExper));
  end
  
  %  Best over the experiments
  [fBestExper(funct),Ind]=min(fExper);
  fMedianExper(funct)=median(fExper);
  fMeanExper(funct)=mean(fExper);
  fStdExper(funct)= std(fExper);
  fWorstExper(funct)=max(fExper);
  fprintf(fidT(funct),'\n%i %1.6e %1.6e %1.6e %1.6e %1.6e  %1.6e ',fun,...
    fBestExper(funct), fMedianExper(funct), fMeanExper(funct),...
    fStdExper(funct), fWorstExper(funct), timeRun);
  
  fprintf(fidRecap,'\n%i %1.6e %1.6e %1.6e %1.6e %1.6e %1.6e ',fun,...
    fBestExper(funct), fMedianExper(funct), fMeanExper(funct),...
    fStdExper(funct), fWorstExper(funct),timeRun);
  
  % Saves the best result
  fprintf(fid(funct),'\nBest position\n');
  for d=1:D
    fprintf(fid(funct),'%1.20e ',xBest(Ind,d));
  end
  
  % Displays the best result
  fprintf('\nBest result %1.6e, mean time %1.6e',fBestExper(funct),timeRun);
  fprintf('\nBest position:\n')
  disp(xBest(Ind,:));
  
  if plotDistrib
    %{
     Plots the distribution of the best results
     and computes some informations about its shape.
     Useful only if the number of experiments is big enough (typically >= 50)
     Note that the data are anyway in the file Exper.txt
    
    	With some strategies that makes an intensive use
    	of randomness, the results are "stabilised"
    	only after at least 1000 experiments
    	So, BE VERY CAREFUL WHEN COMPARING RESULTS
    %}
    fprintf('\n FEmax %i',FEmax);
    fprintf('\nOver experiments: min         median       mean      std. dev.');
    fprintf('\n                  %1.4e  %1.4e  %1.4e %1.4e',...
      fBestExper(funct), fMedianExper(funct), fMeanExper(funct),fStdExper(funct));
    
    %	Plots the distributions of the best results (density and cumulated)
    mini=fBestExper(funct); maxi= fWorstExper(funct);
    CDFplot (fExper,mini,maxi,nbBins,init,reTriang);
  end %  if plotDistrib
  
  % Saves intermediate results (averaged over the number of experiments)
  fprintf(fidI(funct),'\n Intermediate Min \n');
  for interm=1:intermedNb
    u=min(intermedMin(interm,:,funct));
    fprintf(fidI(funct),'%1.6e ',u);
  end
  
  fprintf(fidI(funct),'\n Intermediate Median \n');
  for interm=1:intermedNb
    u=median(intermedMin(interm,:,funct));
    fprintf(fidI(funct),'%1.6e ',u);
  end
  
  if plotAnim
    close(videoFile) % Absolutely needed!
  end
  
  % Save all positions for test purpose (entropy, in particular)
  if savePos
  	FileNameAllPos{funct}=[FileName{funct} 'All pos.txt'];
  	fidAllPos(funct) = fopen(FileNameAllPos{funct}, 'w');
  	[nbPos, D]=size(xTot);
  	
  	for n=1:nbPos
  		for d=1:D
  			fprintf(fidAllPos,'%1.10e ', xTot(n,d));
  		end
  		fprintf(fidAllPos,'%1.10e \n',fTot(n));
  	end
  end
  
end %for funct = 1:nbFunc  -------------------------End of loop on functions (1)
fprintf('\n')
end

