function  [posRand,triRank]=aroundPos(nn,pos,posEstim, tri,around)
	% Samples "around" pos(nn,:)
	
	[nbKnownPos,D]=size(pos);
	[nbEstimPos, ~]=size(posEstim);
	X=pos(nn,:);
	
    switch around
    		case 0 % Rudimentary method
    		delta=1/nbEstimPos;
    		
    		for d=1:D
					rnd=bellLike(-delta,delta,1,5);		
          u=X(d)+rnd;
          posRand(d)=max(min(u,1),0);       
        end
        
        triRank=0;
    
      case 1 % *   % Dimension wise
        % Finds the nearest estimated point  
       [ k,dist] = dsearchn(X,posEstim);
                 
        % and  'exploits ' at random the interval
        for d=1:D
					delta=abs(X(d)-posEstim(k(1),d));
					rnd=bellLike(-delta,delta,1,5);		
          u=X(d)+rnd;
          posRand(d)=max(min(u,1),0);           
        end
        
           triRank=0;
                 
      case 11 % 
        % Along each dimension, finds the nearest point (estimated or known)
        % and  'exploits ' at random the interval
        posTot=[posEstim;pos];
         [ k,dist] = dsearchn(X,posTot);
         
         for d=1:D
					delta=abs(X(d)-posEstim(k(2),d));
					rnd=bellLike(-delta,delta,1,5);		
          u=X(d)+rnd;
          posRand(d)=max(min(u,1),0);           
        end
        
     %{
        [nbPos,~]=size(posTot);
        
        for d=1:D
          posDiff=posTot(:,d)-X(d)*ones(1,nbPos)';
          neigh=sort(posDiff);
          delta=neigh(2); % Because, off course, neigh(1)=0
          %rnd=delta*(-1+2*rand);
          rnd=bellLike(-delta,delta,1,5);	
          u=X(d)+rnd;
          posRand(d)=max(min(u,1),0);
        end
      %}
           triRank=0;
           
      case 12
   
        sigma=0.5/nbKnownPos; % Empirical
        nSum=5;
        
        for d=1:D
          c=bellLike(-sigma,sigma,1,nSum);  % J.  Bell-like perturbation
          posRand(d) = min(1,max(0,X(d)+c));
        end
           triRank=0;
           
      case 13 % * (seems to be good) 
        nSum=5; % (for the bell-like distribution)
        
        for d=1:D % For each dimension ...
        % ... lists the "distances" to the estimated positions ...
          posDiff=posEstim(:,d)-X(d)*ones(1,nbEstimPos)';
        % ... and computes a coefficient depending on their min 

         sigma=abs(min(posDiff)); %/nbEstimPos; 
        % Generates a random number (bell-like distribution)
         c=bellLike(-sigma,sigma,1,nSum); 
        
        % Generates the coordinate. Keeps it inside the search space
          posRand(d) = min(1,max(0,X(d)+c));
        end
        
           triRank=0;
        
     case 131 % Similar to 13, but uses the same estimated position 
     %						for all dimensions
     nSum=5;
     % Finds the nearest distance to estimated positions
     [ind, minDistEstim]=knnsearch(posEstim,X);

     % Computes sigma, and generates the next position
     	sigma=minDistEstim(1);
      
      for d=1:D
      	c=bellLike(-sigma,sigma,1,nSum); 
      	posRand(d) = min(1,max(0,X(d)+c));
      end
        
           triRank=ind(1);
           
      case 14 % Similar to 13, but with the known positions     
      nSum=5; % (for the bell-like distribution)
        
        for d=1:D % For each dimension ...
        % ... lists the distances to the known positions ...
          posDiff=abs(pos(:,d)-X(d)*ones(1,nbKnownPos)');
         posDiff=sort(posDiff);
        % ... and computes a coefficient depending on their min
        %  (except 0, of course, which is for the position itself 
          sigma=posDiff(2);%/nbKnownPos;
           
        % Generates a random number (bell-like distribution)
          c=bellLike(-sigma,sigma,1,nSum); ; % J.  Bell-like perturbation
          
        % Generates the coordinates. Keeps it inside the search space
          posRand(d) = min(1,max(0,X(d)+c));
        end
      
         triRank=0;
      
        
      case 2 % ? % TO MODIFY (cf case 1)
      % Finds the nearest point, and  'exploits '
        % inside a D-rectangle defined by it and the position n
        neighDistMin=Inf;
        posTot=[posEstim;pos];
         [nbPos,~]=size(posTot);
         
        for s=1:nbPos
          if s==nn   continue; end
          neighDist=pdist([X;posTot(s,:)]); % [distance(pos(n,:),posTot(s,:));
          if neighDist<neighDistMin
            neighDistMin=neighDist;
            neighMin=s;
          end
        end
        for d=1:D
          delta=posTot(neighMin,d)-X(d);
          rnd=delta*(-1+2*rand);
          u=X(d)+rnd;
          posRand(d)=max(0,min(u,1));
        end
        
           triRank=0;
           
      case 3  % ? (invariant by rotation)
        % Finds all the facets it belongs to
        [nbTri,~]=size(tri);
        
        nbBelong=0;
        for t=1:nbTri
          if ismember(nn,tri(t,:))
            nbBelong=nbBelong+1;
            belong(nbBelong)=t;
          end
        end
        
        %  'belong ' contains the ranks of the facets in  'tri '
        if nbBelong==1
          % If just one facet, draws at random inside it
          for b=1:D+1
            rd(b)=rand;
          end
          rd=rd/sum(rd); % Normalises
          
          posRand(1:D)=[];
          for b=1:D+1
            posRand(1:D)=posRand+rd(b)*pos(tri(belong,b),:);
          end
          
        else
          % Defines the polyedron of their centres
          % and draws at random inside it
          for b=1:nbBelong
            centre(b,1:D)=mean(pos(tri(belong(b),:),:));
          end
          
          for b=1:nbBelong
            rd(b)=rand;
          end
          
          rd=rd/sum(rd); % Normalises
          posRand=zeros(1:D);
          for b=1:nbBelong
            posRand=posRand+rd(b)*centre(b,:);
          end
        end  %if nbBelong==1
        
           triRank=0;
        
      case 4 %*
        % Finds a facet it belongs to
        [nbTri,dum]=size(tri);
        rndPerm=randperm(nbTri);
        
        for t=1:nbTri
        	tt=rndPerm(t);
          if ismember(nn,tri(tt,:))
						break;
          end
        end
        
        % Selects a point at random  'between ' the position
        % and the representative of the facet
        delta=posEstim(tt,:)-X;
       % posRand(1:D)=pos(nn,:)+delta.*rand(1,D);
       
       rnd=rand;
       posRand(1:D)=X+delta*rnd;
        triRank=tt; % Is inside this facet      
        
      case 41 % 
        % Finds the first facet it belongs to
        [nbTri,dum]=size(tri);
        
        for t=1:nbTri
          if ismember(nn,tri(t,:))
						break;
          end
        end
        
        % Selects a point at random  'between ' the position
        % and the representative of the facet
        delta=posEstim(t,:)-X;
 
        rnd=rand;
       posRand(1:D)=X+delta*rnd;
        triRank=t; % Is inside this facet 
        
    end % switch around
end
